// CScriptTrigger trigger script samples for Trespasser CE

/*
A script based custom trigger that fires when specified creature(s) die. It's modeled after the standard CCreatureTrigger,
except this also supports firing only when all creatures are dead and this script only deals with creature deaths.

TScript:
{
    string Class = "CScriptTrigger"
    string Script = "CreatureDieTrig"
    string A00 = "Raptor00-01"
    string A01 = "Raptor00-02"              (optional)
    ...
    string A07 = "Raptor00-08"              (optional)
    bool EvaluateAll = true                 (optional) if true then all creatures must be dead to fire trigger

    ...
}
*/
class CreatureDieTrig extends CScriptTrigger
{
	// creature list/array
	au4Creatures = null;

	// if true, fire trigger only if all creatures in list are dead
	bEvaluateAll = null;

	constructor()
	{
		au4Creatures = [];
		bEvaluateAll = false;

		// init trigger from TScript params

		// support up to 8 creatures
		for (local u4_count=0; u4_count<8; u4_count++)
		{
			local sName = GetPropString(esA00 + u4_count);
			if (sName == null)
				break;

			au4Creatures.push( TP_Engine.uid(sName) );
		}

		bEvaluateAll = GetPropBool(esEvaluateAll, bEvaluateAll);
	}

	// returns true if creature handle is found in list
	function bIsInList(u4Handle)
	{
		foreach (i in au4Creatures)
			if (i == u4Handle)
				return true;
		return false;
	}

	function OnProcessDeath(pantDyingThing, pinsCorpse)
	{
		// if the dead creature isn't in the list then it's irrelevant to this trigger
		if ( !bIsInList( pantDyingThing.u4GetUniqueHandle() ) )
			return;

		if (bEvaluateAll)
		{
			// only fire trigger if all creatures in list are dead
			foreach (i in au4Creatures)
			{
				local pins = TP_Engine.FindInstance(i);
				if ((pins instanceof CAnimate) && !pins.bDead())
					// found a live one, don't fire trigger
					return;
			}
		}

		AttemptTriggerFire();
	}

	function bEvaluateNow()
	{
		if (bEvaluateAll)
		{
			// all creatures in list have to be dead
			foreach (i in au4Creatures)
			{
				local pins = TP_Engine.FindInstance(i);
				if ((pins instanceof CAnimate) && !pins.bDead())
					// found a live one
					return false;
			}

			// all are dead
			return true;
		}
		else
		{
			// at least one creature in list has to be dead
			foreach (i in au4Creatures)
			{
				local pins = TP_Engine.FindInstance(i);
				if ((pins instanceof CAnimate) && pins.bDead())
					// found a dead one
					return true;
			}

			// all are alive
			return false;
		}
	}
}


/*
A script based custom trigger that fires when all instance have moved inside are and/or all have moved outside

enter behavior:
    trigger fires when the last instance enters the trigger volume,
	it also fires if one (or some) but not all leave and then enter the trigger volume again

leave behavior:
    trigger fires after the last instance leaves the trigger volume, ONLY if all instances were previously inside
	after the trigger fires, it once again requires all instances to be inside before it can fire when the last leaves

if the trigger is a one-time trigger with both enter and leave triggering enabled, only the enter event will fire
if an action type sequence is still executing while the trigger wants to fire again, it will not fire

TScript:
{
    string Class = "CScriptTrigger"
    string Script = "LocAllInOutTrig"
    string A00 = "Raptor00-01"
    string A01 = "Raptor00-02"              (optional)
    ...
    string A07 = "Raptor00-08"              (optional)
    bool PointTrigger = false               (optional) if false the instance bounding volumes are used for intersection test with trigger, otherwise only their pos
    bool ObjectEnterTrigger = true          (optional) if true the trigger fires when all instances have entered and are inside the trigger volume
    bool ObjectLeaveTrigger = true          (optional) if true the trigger fires when all instances have left the trigger volume, after all have been inside at once

    ...
}
*/
class LocAllInOutTrig extends CScriptTrigger
{
	// nvram for savegame data
	nvram = null;

	// if true then test instance position point inside trigger volume, otherwise do a volume vs volume test
	bPointTrigger = null;

	// trigger when all instances are inside
	bTriggerInside = null;
	// trigger when all instances have left
	bTriggerOutside = null;

	// array of trigger activating instances
	apinsActivate = null;

	constructor()
	{
		nvram = {};
		// array of instances currently inside trigger
		nvram.apinsContained <- [];
		// set to true once all instances are inside (used to determine if the leave-logic is allowed to kick in)
		nvram.bAllHaveBeenInside <- false;

		bPointTrigger = true;
		bTriggerInside = false;
		bTriggerOutside = false;
		apinsActivate = [];

		// init trigger from TScript params

		for (local u4_count=0; u4_count<8; u4_count++)
		{
			local sName = GetPropString(esA00 + u4_count);
			if (sName == null)
				break;

			local pins = TP_Engine.FindUnmappedInstance(sName);
			if (pins)
			{
				apinsActivate.push(pins);
				// also add the instance to the CScriptTrigger filter list, so that we get OnProcessMove events for the instances
				AddFilterInstance( pins.u4GetUniqueHandle() );
			}
		}

		bPointTrigger = GetPropBool(esLTPointTrigger, bPointTrigger);
		bTriggerInside = GetPropBool(esLTObjectEnter, bTriggerInside);
		bTriggerOutside = GetPropBool(esLTObjectLeave, bTriggerOutside);
	}

	function OnScriptInit(bSaveGame)
	{
		if (!bSaveGame)
		{
			// reset state
			nvram.apinsContained.clear();
			nvram.bAllHaveBeenInside = false;
		}
	}

	function OnProcessMove(etType, pinsMover, v3PosPrev, r3RotPrev, sWhen)
	{
		if (etType == EMoveType.etMOVED && bEvaluate(pinsMover))
			AttemptTriggerFire();
	}

	function bEvaluteNow()
	{
		if (bTriggerInside && nvram.apinsContained.len() == apinsActivate.len())
			return true;
		if (bTriggerOutside && nvram.bAllHaveBeenInside && !nvram.apinsContained.len())
			return true;
		return false;
	}

	function bEvaluate(pins)
	{
		local b_inside = bContained(pins);
		local b_in = bIntersectsTriggerVolume(pins, bPointTrigger);
		
		// check if it transitioned from outside in or vice versa
		if (b_inside == b_in)
			return false;

		if (!b_inside)
		{
			// instance entered the trigger volume

			SetContained(pins);

			if (nvram.apinsContained.len() == apinsActivate.len())
			{
				// all instances are inside, make sure bAllHaveBeenInside is set
				nvram.bAllHaveBeenInside = true;

				if (bTriggerInside)
					return true;
			}
		}
		else
		{
			// instance left the trigger volume

			RemoveContained(pins);

			if (bTriggerOutside && nvram.bAllHaveBeenInside && !nvram.apinsContained.len())
			{
				// reset bAllHaveBeenInside because for the outside trigger to fire again all instances have to once again first enter the trigger
				nvram.bAllHaveBeenInside = false;
				return true;
			}
		}

		return false;
	}

	function bContained(pins)
	{
		foreach (i in nvram.apinsContained)
			if (i == pins)
				return true;
		return false;
	}

	function SetContained(pins)
	{
		nvram.apinsContained.push(pins);
	}

	function RemoveContained(pins)
	{
		foreach (idx, i in nvram.apinsContained)
			if (i == pins)
			{
				nvram.apinsContained.remove(idx);
				return;
			}
	}
}
